@@ -47,8 +47,8 @@ pub enum BibLaTeXError {
 impl std::fmt::Display for BibLaTeXError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
-            Self::Parse(err) => write!(f, "biblatex parse error: {}", err),
-            Self::Type(err) => write!(f, "biblatex type error: {}", err),
+            Self::Parse(err) => write!(f, "biblatex parse error: {err}"),
+            Self::Type(err) => write!(f, "biblatex type error: {err}"),
         }
     }
 }
