@@ -8,11 +8,11 @@ use std::{fmt, fs};
 
 mod common;
 use citationberg::taxonomy::Locator;
-use citationberg::{Locale, LocaleCode, Style, XmlError};
-use common::{ensure_repo, iter_files_with_name, CACHE_PATH};
+use citationberg::{Locale, LocaleCode, Style, XmlDeError};
+use common::{CACHE_PATH, ensure_repo, iter_files_with_name};
 
 use citationberg::json as csl_json;
-use hayagriva::archive::{locales, ArchivedStyle};
+use hayagriva::archive::{ArchivedStyle, locales};
 use hayagriva::io::from_biblatex_str;
 use hayagriva::{
     BibliographyDriver, BibliographyRequest, CitationItem, CitationRequest, CitePurpose,
@@ -107,21 +107,21 @@ enum TestParseError {
     SyntaxError,
     WrongClosingTag,
     MissingRequiredSection(SectionTag),
-    CslError(XmlError),
+    CslError(XmlDeError),
     JsonError(serde_json::Error),
 }
 
 impl fmt::Display for TestParseError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
-            TestParseError::UnknownSection(s) => write!(f, "unknown section {}", s),
+            TestParseError::UnknownSection(s) => write!(f, "unknown section {s}"),
             TestParseError::SyntaxError => write!(f, "syntax error"),
             TestParseError::WrongClosingTag => write!(f, "wrong closing tag"),
             TestParseError::MissingRequiredSection(s) => {
-                write!(f, "missing required section {}", s)
+                write!(f, "missing required section {s}")
             }
-            TestParseError::CslError(e) => write!(f, "csl error: {}", e),
-            TestParseError::JsonError(e) => write!(f, "json error: {}", e),
+            TestParseError::CslError(e) => write!(f, "csl error: {e}"),
+            TestParseError::JsonError(e) => write!(f, "json error: {e}"),
         }
     }
 }
@@ -318,7 +318,7 @@ fn test_parse_tests() {
     );
     let percentage =
         results.passed.len() as f64 / (results.total - results.skipped) as f64 * 100.0;
-    eprintln!("{:.2}% passed", percentage);
+    eprintln!("{percentage:.2}% passed");
 
     let should_pass = load_passing_tests();
     // Enable binary search.
@@ -331,9 +331,9 @@ fn test_parse_tests() {
     for test in &should_pass {
         if results.passed.binary_search(test).is_err() {
             if results.failed.binary_search(test).is_ok() {
-                eprintln!("❌ Test {} should pass but failed", test);
+                eprintln!("❌ Test {test} should pass but failed");
             } else {
-                eprintln!("🤨 Test {} should pass but was not found", test);
+                eprintln!("🤨 Test {test} should pass but was not found");
             }
 
             fail = true;
@@ -349,12 +349,14 @@ fn test_parse_tests() {
 
     // Check that the `passing` file contains all passed tests.
     for test in &results.passed {
-        eprintln!("👁️ Test {} passed but is not in the `passing` file", test);
+        eprintln!("👁️ Test {test} passed but is not in the `passing` file");
         fail = true;
     }
 
     if fail {
-        panic!("⚠️ Some test passed but were not found in the `passing` file.\nRebuild the file by running `cargo test --test citeproc --features csl-json -- write_passing --ignored`")
+        panic!(
+            "⚠️ Some test passed but were not found in the `passing` file.\nRebuild the file by running `cargo test --test citeproc --features csl-json -- write_passing --ignored`"
+        )
     }
 
     eprintln!("✅ All tests expected to pass passed");
@@ -432,7 +434,7 @@ impl TestSuiteResults {
 #[ignore]
 fn test_single_file() {
     let locales = locales();
-    let name = "nameattr_DelimiterPrecedesLastOnCitationInCitation.txt";
+    let name = "bugreports_ArabicLocale.txt";
     let test_path = PathBuf::from(CACHE_PATH)
         .join(TEST_REPO_NAME)
         .join("processor-tests/humans/");
@@ -473,7 +475,7 @@ where
     let can_test = case.bib_entries.is_none()
         && case.bib_section.is_none()
         && case.citations.is_none()
-        && case.citation_items.as_ref().map_or(true, |cites| {
+        && case.citation_items.as_ref().is_none_or(|cites| {
             cites.iter().flatten().all(|i| {
                 i.prefix.is_none()
                     && i.suffix.is_none()
@@ -488,8 +490,7 @@ where
         .flat_map(|i| i.0.values())
         .filter_map(|v| if let csl_json::Value::Date(d) = v { Some(d) } else { None })
         .any(|d| {
-            csl_json::FixedDateRange::try_from(d.clone())
-                .map_or(false, |d| d.end.is_some())
+            csl_json::FixedDateRange::try_from(d.clone()).is_ok_and(|d| d.end.is_some())
         });
 
     if !can_test {
@@ -615,7 +616,7 @@ where
     } else {
         eprintln!("Test {} failed", display());
         eprintln!("Expected:\n{}", case.result);
-        eprintln!("Got:\n{}", output);
+        eprintln!("Got:\n{output}");
         false
     }
 }
@@ -625,7 +626,9 @@ where
 mod citeproc_bib {
     use core::fmt;
 
-    use citationberg::{Display, FontStyle, FontVariant, FontWeight, VerticalAlign};
+    use citationberg::{
+        Display, FontStyle, FontVariant, FontWeight, TextDecoration, VerticalAlign,
+    };
     use hayagriva::{BufWriteFormat, Elem, ElemChild, Formatting};
 
     pub(super) fn render(
@@ -741,6 +744,13 @@ mod citeproc_bib {
             FontVariant::Normal => {}
         }
 
+        match formatting.text_decoration {
+            // NOTE: No existing citeproc tests use this, so this is guesswork.
+            // However, we can use this in local tests.
+            TextDecoration::Underline => push_elem("<u>", "</u>"),
+            TextDecoration::None => {}
+        }
+
         if !css.is_empty() {
             push_elem(&format!("<span style=\"{css}\">"), "</span>");
         }
@@ -901,7 +911,7 @@ fn case_folding() {
         .content
         .write_buf(&mut buf, hayagriva::BufWriteFormat::Plain)
         .unwrap();
-    assert_eq!(buf, ". my lowercase container title.");
+    assert_eq!(buf, "my lowercase container title.");
 }
 
 #[test]
@@ -979,7 +989,10 @@ fn no_author() {
         .write_buf(&mut buf, hayagriva::BufWriteFormat::Plain)
         .unwrap();
 
-    assert_eq!(buf, "Definition and objectives of systems development. (2016, January 19). https://www.opentextbooks.org.hk/ditatopic/25323");
+    assert_eq!(
+        buf,
+        "Definition and objectives of systems development. (2016, January 19). https://www.opentextbooks.org.hk/ditatopic/25323"
+    );
 
     let mut buf = String::new();
     rendered.citations[0]
